package org.jboss.eap.util.xp.patch.stream.manager;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Set;

/**
 * @author <a href="mailto:kabir.khan@jboss.com">Kabir Khan</a>
 */
public abstract class ManagerAction {

    protected final ManagerStatus status;
    private final boolean supportPolicyAccepted;

    public ManagerAction(ManagerStatus status, boolean supportPolicyAccepted) {
        this.status = status;
        this.supportPolicyAccepted = supportPolicyAccepted;
    }

    abstract ManagerState getToManagerState();

    abstract ManagerCommand getCommand();

    abstract void doExecute() throws Exception;

    final void execute() throws Exception {
        if (getToManagerState() != null) {
            Set<ManagerCommand> availableCommands = status.getAvailableCommands();
            if (!availableCommands.contains(getCommand())) {
                System.err.println(ManagerLogger.LOGGER.commandNotAvailableInCurrentContext(getCommand()));
                System.exit(1);
            }
        }
        acceptSupportLevel();
        doExecute();
    }

    private void acceptSupportLevel() throws IOException {
        ManagerState toState = getToManagerState();
        if (toState != ManagerState.CLEAN && toState != ManagerState.INSTALLED) {
            return;
        }
        System.out.println();
        if (toState == ManagerState.INSTALLED) {
            System.out.println(ManagerLogger.LOGGER.xpSupportPolicy());
        } else {
            System.out.println(ManagerLogger.LOGGER.xpRemoveSupportPolicy());
        }

        if (supportPolicyAccepted) {
            System.out.println();
            System.out.println(ManagerLogger.LOGGER.supportPolicyAcceptedWithArg());
            return;
        }

        boolean accepted = false;

        while (!accepted) {
            System.out.println();
            System.out.print(ManagerLogger.LOGGER.acceptSupportPolicy() + " ");

            BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
            String answer = reader.readLine().toLowerCase();
            if (answer.equals("yes") || answer.equals("y")) {
                accepted = true;
                System.out.println();
            }  else if (answer.equals("no") || answer.equals("n")) {
                System.exit(0);
            } else {
                System.out.println(ManagerLogger.LOGGER.invalidSupportPolicyResponse());
            }
        }
    }
}
